# Script to transfer WX chart files via http
# ------------------------------------------
# v1.0.1 050201
# my mods 20050719 Gtz Romahn -- GWR; goetz@g-romahn.de 
#
# Typically run automatically as a scheduled task every 6 hours.
#
# Usage: brackachart.pl [<directory to place files> [<required file> [<address of server>]]]
# e.g.   brackabhart.pl C:\MSG-1\Images\Hrit BRACK.GIF http://www.wetterzentrale.de/pics
# commandline arguments must be specified from left to right
# address of server defaults to	http://www.wetterzentrale.de/pics
# required file			BRACKA.GIF
# directory to place files	.
# Thus a typical commandline would look like this: brackachart.pl C:\MSG-1\Images\Hrit
#
#
# Files have a timestamp appended in the form _yymmdd_hhmm, e.g. bracka_050124_1800.gif
# and are placed in subdirectories giving the date the the usual format (\yyyy\mm\dd) under <directory to place files>
# e.g. C:\MSG-1\Images\Hrit\2005\01\24\bracka_050124_1800.gif
# Directories will be created as necessary.
#
# The hhmm time component is generated based on a chart which is updated every 6 hours, and should equate to the
# issue time of the chart. <<< THIS MAY NEED CHAGING DEPENDING ON THE FREQUENCY OF ISSUE OF THE CHART >>>
#
# The best time to schedule the job is 1 hour before the end of the chart's validity, i.e.
# at 05:00 for the 00:00 chart
# at 11:00 for the 06:00 chart
# at 17:00 for the 12:00 chart
# at 23:00 for the 18:00 chart
# This allows time for the chart to be late getting to the server as well as for the transfer to be
# retried if it fails initially.
#
# If the transfer fails it will be retried at 5 min intervals for 5 attempts, before giving up.
#
# Francis Breame G8ISI 
# vf0123@btinternet.com


use LWP;
use HTTP::request;
use HTTP::response;
use File::Path;

my $defHost	= "http://www.wetterzentrale.de/pics",
   $defFile	= "bracka.gif",
   $defDirBase	= ".",
   $toDirBase,
   $fromFile,
   $maxTries = 5,
   $waitSecs = 300;

my $toDir,
   $toFile,
   $toPath,
   $tries,
   $t = 0,
   $ftp,
   $size,
   $n;

$toDirBase = shift(@ARGV);		# e.g. C:\MSG-1\Images\Hrit
$fromFile  = shift(@ARGV);		# e.g. bracka.gif
$host      = shift(@ARGV);		# Default is http://www.wetterzentrale.de/pics


sub getTimestamp
{
	my $s,$mi,$h,$d,$m,$y;
	($s,$mi,$h,$d,$m,$y) = localtime(time);
	return sprintf "%04d/%02d/%02d %02d:%02d:%02d: ", $y+1900,$m+1,$d,$h,$mi,$s;
}

sub pad2
{
	return sprintf "%02d", shift(@_);
}

sub pad4
{
	return sprintf "%04d", shift(@_);
}

sub mes
{
	my $s = shift;
	#$s=~s/\n//s;
	print getTimestamp, $s, "\n";
}

sub doHTTP
{
	my $ua, $request, $response, $data, $res;
	my $url = $host."/".$fromFile;

	mes "Getting from '$url' to '$toPath'";

	$ua = LWP::UserAgent->new;
	$request = HTTP::Request->new(GET => $url);
	$response = $ua->request($request); 

	if ($response->is_success)
	{
  		open OUT, ">".$toPath or die "Can't open output file $toPath\n";
		binmode OUT;
		$data = $response->content_ref; 
		print OUT $$data;
		$res = "succeeded";
	}
	else
	{
		$res = "failed";
	}

	mes "HTTP request to '$url' $res";
	mes "Status = ".$response->status_line;
	mes "Header info:\n".$response->headers_as_string;

	close OUT if (defined OUT);
}



# Default args
if (!defined $toDirBase)
{
	$toDirBase = $defDirBase;
}
if (!defined $fromFile)
{
	$fromFile = $defFile;
}
if (!defined $host)
{
	$host = $defHost;
}

mes "------------------------------------------------";
mes "Running under account '".getlogin."'";

($s,$mi,$h,$d,$m,$y) = localtime(time);
$y2 = pad2 $y-100;
$y4 = pad4 $y+1900;
$m  = pad2 $m+1;
$d  = pad2 $d;
$h  = pad2 $h;
$t  = pad2 $t;
$toDir = "$toDirBase\\$y4\\$m\\$d";
if (!-f $toDir)
{
	$n=0;
	mkpath $toDir;
}

if    ($h>21 || $h<=3) {$t = "1800";}
elsif ($h>15) {$t = "1200";}
elsif ($h>9)  {$t = "0600";}
else          {$t = "0000";}

$toFile = $fromFile;
$toFile =~ s/\./_$y2$m$d\_$t\./;
$toPath = $toDir."\\".$toFile;

$tries = $maxTries;

if (-f $toPath)
{
	mes "File '$toPath' already exists";
	exit;
}

while (!-f $toPath && $tries>0)
{
	if ($tries != $maxTries)
	{
		mes "Waiting for $waitSecs secs...";
		sleep $waitSecs;
		mes "Having another go - $tries to go";
	}
	$tries--;
	doHTTP;
}

if (!-f $toPath && $tries<=0) {mes "Gave up"};

